# $Id: green.tcl 96 2008-02-12 01:59:16Z kostix $
# "Green" -- Tkabber plugin which allows Tkabber to properly
# handle changes in the system power state (disconnection on
# suspend and (possibly) re-connection on resume).
# On Windows it requires winpm Tcl extension: http://tkwinpm.googlecode.com
# Written by Konstantin Khomoutov <flatworm@users.sourceforge.net>
# See README for usage guidelines.
# See license.terms for details about distribution.

namespace eval green {
	if {[string equal $::tcl_platform(platform) windows]} {
		if {[catch { package require winpm }]} {
			global errorInfo
			puts stderr "Power management support is unavailable:\
				failed to load \"winpm\" package.\
				The error was:\n\
				$errorInfo"
			return
		}

		winpm bind PBT_APMSUSPEND [namespace current]::on_suspend
		winpm bind PBT_APMRESUMESUSPEND [namespace current]::on_resume
	}

	variable active [list]
	variable options

	custom::defgroup Green \
		[::msgcat::mc "Power management support."] \
		-group Plugins

	custom::defvar options(resume_sessions) 1 \
		[::msgcat::mc "Attempt to resume the sessions that were active\
			when the system has been suspended."] \
		-type boolean \
		-group Green

	custom::defvar options(resume_delay) 10 \
		[::msgcat::mc "Number of seconds to wait before attempting\
			to re-establish the sessions that were active.\
			This may be useful to allow the network connection(s)\
			to come up."] \
		-type integer \
		-group Green
}

proc green::on_suspend {} {
	variable options

	set connids [jlib::connections]
	if {[llength $connids] == 0} return

	variable active [list]
	foreach connid $connids {
		if {$options(resume_sessions)} {
			set jid [jlib::connection_bare_jid $connid]
			lappend active $jid
		}
		logout $connid
	}
}

proc green::on_resume {} {
	variable options
	variable active

	if {!$options(resume_sessions)} return
	if {[llength $active] == 0} return

	after idle after [expr {$options(resume_delay) * 1000}] \
		[namespace current]::do_resume
}

proc green::do_resume {} {
	variable active

	set lcs [lsearch -all -inline -regexp [info vars ::*] ::loginconf\\d+]
	if {[llength $lcs] == 0} {
		upvar 0 ::loginconf lc
		set jid $lc(user)@$lc(server)
		if {[lsearch -exact $active $jid] >= 0 && ![jid_connected $jid]} {
			login [array get ::loginconf]
		}
	} else {
		foreach vlc $lcs {
			upvar 0 $vlc lc
			if {![info exists lc(user)]
					|| ![info exists lc(server)]} continue
			set jid $lc(user)@$lc(server)
			foreach saved $active {
				if {[string equal $saved $jid] && ![jid_connected $jid]} {
					login [concat [array get ::loginconf] [array get lc]]
				}
			}
		}
	}
}

proc green::jid_connected jid {
	set ref [node_and_server_from_jid $jid]
	foreach connid [::jlib::connections] {
		set cjid [::jlib::connection_bare_jid $connid]
		if {[string equal $ref $cjid]} {
			return 1
		}
	}
	return 0
}

